@@ -1,46 +1,73 @@
 #include "HttpHandler.h"
 
-#include "hbase.h"
+#include "hversion.h"
 #include "herr.h"
 #include "hlog.h"
 #include "htime.h"
 #include "hurl.h"
 #include "hasync.h" // import hv::async for http_async_handler
+
+#include "httpdef.h"
+#include "http2def.h"
+#include "wsdef.h"
+
 #include "http_page.h"
 
 #include "EventLoop.h" // import hv::setInterval
 using namespace hv;
 
-HttpHandler::HttpHandler() {
-    protocol = UNKNOWN;
-    state = WANT_RECV;
-    ssl = false;
-    keepalive = true;
-    proxy = false;
-    service = NULL;
-    ws_service = NULL;
-    api_handler = NULL;
-    last_send_ping_time = 0;
-    last_recv_pong_time = 0;
+#define MIN_HTTP_REQUEST        "GET / HTTP/1.1\r\n\r\n"
+#define MIN_HTTP_REQUEST_LEN    14 // exclude CRLF
 
-    files = NULL;
-    file = NULL;
+#define HTTP_100_CONTINUE_RESPONSE      "HTTP/1.1 100 Continue\r\n\r\n"
+#define HTTP_100_CONTINUE_RESPONSE_LEN  25
+#define HTTP_200_CONNECT_RESPONSE       "HTTP/1.1 200 Connection established\r\n\r\n"
+#define HTTP_200_CONNECT_RESPONSE_LEN   39
+
+HttpHandler::HttpHandler(hio_t* io) :
+    protocol(HttpHandler::UNKNOWN),
+    state(WANT_RECV),
+    error(0),
+    // flags
+    ssl(0),
+    keepalive(1),
+    upgrade(0),
+    proxy(0),
+    proxy_connected(0),
+    forward_proxy(0),
+    reverse_proxy(0),
+    ip{'\0'},
+    port(0),
+    pid(0),
+    tid(0),
+    // for http
+    io(io),
+    service(NULL),
+    api_handler(NULL),
+    // for websocket
+    ws_service(NULL),
+    last_send_ping_time(0),
+    last_recv_pong_time(0),
+    // for sendfile
+    files(NULL),
+    file(NULL),
+    // for proxy
+    proxy_port(0)
+{
+    // Init();
 }
 
 HttpHandler::~HttpHandler() {
-    closeFile();
-    if (writer) {
-        writer->status = hv::SocketChannel::DISCONNECTED;
-    }
+    Close();
 }
 
-bool HttpHandler::Init(int http_version, hio_t* io) {
+bool HttpHandler::Init(int http_version) {
     parser.reset(HttpParser::New(HTTP_SERVER, (enum http_version)http_version));
     if (parser == NULL) {
         return false;
     }
-    req.reset(new HttpRequest);
-    resp.reset(new HttpResponse);
+    req  = std::make_shared<HttpRequest>();
+    resp = std::make_shared<HttpResponse>();
     if(http_version == 1) {
         protocol = HTTP_V1;
     } else if (http_version == 2) {
@@ -49,15 +76,41 @@ bool HttpHandler::Init(int http_version, hio_t* io) {
         resp->http_minor = req->http_minor = 0;
     }
     if (io) {
-        writer.reset(new hv::HttpResponseWriter(io, resp));
+        hloop_t* loop = hevent_loop(io);
+        pid = hloop_pid(loop);
+        tid = hloop_tid(loop);
+        writer = std::make_shared<HttpResponseWriter>(io, resp);
         writer->status = hv::SocketChannel::CONNECTED;
+    } else {
+        pid = hv_getpid();
+        tid = hv_gettid();
     }
-    initRequest();
+    parser->InitRequest(req.get());
+    // NOTE: hook http_cb
+    req->http_cb = [this](HttpMessage* msg, http_parser_state state, const char* data, size_t size) {
+        if (this->state == WANT_CLOSE) return;
+        switch (state) {
+        case HP_HEADERS_COMPLETE:
+            if (this->error != 0) return;
+            onHeadersComplete();
+            break;
+        case HP_BODY:
+            if (this->error != 0) return;
+            onBody(data, size);
+            break;
+        case HP_MESSAGE_COMPLETE:
+            onMessageComplete();
+            break;
+        default:
+            break;
+        }
+    };
     return true;
 }
 
 void HttpHandler::Reset() {
     state = WANT_RECV;
+    error = 0;
     req->Reset();
     resp->Reset();
     ctx = NULL;
@@ -68,27 +121,35 @@ void HttpHandler::Reset() {
         writer->onwrite = NULL;
         writer->onclose = NULL;
     }
-    initRequest();
+    parser->InitRequest(req.get());
 }
 
-void HttpHandler::initRequest() {
-    parser->InitRequest(req.get());
-    // NOTE: hook http_cb
-    req->http_cb = [this](HttpMessage* msg, http_parser_state state, const char* data, size_t size) {
-        if (state == HP_HEADERS_COMPLETE) {
-            onHeadersComplete();
-        } else if (state == HP_BODY) {
-            if (api_handler && api_handler->state_handler) {
-                // goto cb;
-            } else {
-                msg->body.append(data, size);
-            }
+void HttpHandler::Close() {
+    if (writer) {
+        writer->status = hv::SocketChannel::DISCONNECTED;
+    }
+
+    if (api_handler && api_handler->state_handler) {
+        if (parser && !parser->IsComplete()) {
+            api_handler->state_handler(context(), HP_ERROR, NULL, 0);
         }
-        if (api_handler && api_handler->state_handler) {
-// cb:
-            api_handler->state_handler(getHttpContext(), state, data, size);
+        return;
+    }
+
+    // close proxy
+    closeProxy();
+
+    // close file
+    closeFile();
+
+    // onclose
+    if (protocol == HttpHandler::WEBSOCKET) {
+        WebSocketOnClose();
+    } else {
+        if (writer && writer->onclose) {
+            writer->onclose();
         }
-    };
+    }
 }
 
 bool HttpHandler::SwitchHTTP2() {
@@ -103,13 +164,12 @@ bool HttpHandler::SwitchHTTP2() {
     return true;
 }
 
-bool HttpHandler::SwitchWebSocket(hio_t* io) {
-    if (!io && writer) io = writer->io();
+bool HttpHandler::SwitchWebSocket() {
     if(!io) return false;
 
     protocol = WEBSOCKET;
-    ws_parser.reset(new WebSocketParser);
-    ws_channel.reset(new hv::WebSocketChannel(io, WS_SERVER));
+    ws_parser  = std::make_shared<WebSocketParser>();
+    ws_channel = std::make_shared<WebSocketChannel>(io, WS_SERVER);
     ws_parser->onMessage = [this](int opcode, const std::string& msg){
         ws_channel->opcode = (enum ws_opcode)opcode;
         switch(opcode) {
@@ -119,7 +179,7 @@ bool HttpHandler::SwitchWebSocket(hio_t* io) {
         case WS_OPCODE_PING:
             // printf("recv ping\n");
             // printf("send pong\n");
-            ws_channel->sendPong();
+            ws_channel->send(msg, WS_OPCODE_PONG);
             break;
         case WS_OPCODE_PONG:
             // printf("recv pong\n");
@@ -154,7 +214,7 @@ bool HttpHandler::SwitchWebSocket(hio_t* io) {
     return true;
 }
 
-const HttpContextPtr& HttpHandler::getHttpContext() {
+const HttpContextPtr& HttpHandler::context() {
     if (!ctx) {
         ctx = std::make_shared<hv::HttpContext>();
         ctx->service = service;
@@ -180,14 +240,100 @@ int HttpHandler::invokeHttpHandler(const http_handler* handler) {
         status_code = HTTP_STATUS_NEXT;
     } else if (handler->ctx_handler) {
         // NOTE: ctx_handler run on IO thread, you can easily post HttpContextPtr to your consumer thread for processing.
-        status_code = handler->ctx_handler(getHttpContext());
+        status_code = handler->ctx_handler(context());
     } else if (handler->state_handler) {
-        status_code = resp->status_code;
+        status_code = handler->state_handler(context(), HP_MESSAGE_COMPLETE, NULL, 0);
     }
     return status_code;
 }
 
 void HttpHandler::onHeadersComplete() {
+    // printf("onHeadersComplete\n");
+    handleRequestHeaders();
+
+    HttpRequest* pReq = req.get();
+    if (service && service->pathHandlers.size() != 0) {
+        service->GetRoute(pReq, &api_handler);
+    }
+
+    if (api_handler && api_handler->state_handler) {
+        api_handler->state_handler(context(), HP_HEADERS_COMPLETE, NULL, 0);
+        return;
+    }
+
+    if (proxy) {
+        handleProxy();
+        return;
+    }
+
+    // Expect: 100-continue
+    handleExpect100();
+}
+
+void HttpHandler::onBody(const char* data, size_t size) {
+    if (api_handler && api_handler->state_handler) {
+        api_handler->state_handler(context(), HP_BODY, data, size);
+        return;
+    }
+
+    if (proxy && proxy_connected) {
+        if (io) hio_write_upstream(io, (void*)data, size);
+        return;
+    }
+
+    req->body.append(data, size);
+    return;
+}
+
+void HttpHandler::onMessageComplete() {
+    // printf("onMessageComplete\n");
+    int status_code = HTTP_STATUS_OK;
+
+    if (error) {
+        SendHttpStatusResponse(resp->status_code);
+        return;
+    }
+
+    if (proxy) {
+        if (proxy_connected) Reset();
+        return;
+    }
+
+    addResponseHeaders();
+
+    // upgrade ? handleUpgrade : HandleHttpRequest
+    if (upgrade) {
+        auto iter_upgrade = req->headers.find("upgrade");
+        if (iter_upgrade != req->headers.end()) {
+            handleUpgrade(iter_upgrade->second.c_str());
+            status_code = resp->status_code;
+        }
+    } else {
+        status_code = HandleHttpRequest();
+        if (status_code != HTTP_STATUS_NEXT) {
+            SendHttpResponse();
+        }
+    }
+
+    // access log
+    if (service && service->enable_access_log) {
+        hlogi("[%ld-%ld][%s:%d][%s %s]=>[%d %s]",
+            pid, tid, ip, port,
+            http_method_str(req->method), req->path.c_str(),
+            resp->status_code, resp->status_message());
+    }
+
+    if (status_code != HTTP_STATUS_NEXT) {
+        // keepalive ? Reset : Close
+        if (keepalive) {
+            Reset();
+        } else {
+            state = WANT_CLOSE;
+        }
+    }
+}
+
+void HttpHandler::handleRequestHeaders() {
     HttpRequest* pReq = req.get();
     pReq->scheme = ssl ? "https" : "http";
     pReq->client_addr.ip = ip;
@@ -196,11 +342,28 @@ void HttpHandler::onHeadersComplete() {
     // keepalive
     keepalive = pReq->IsKeepAlive();
 
-    // NOTE: Detect proxy before ParseUrl
-    proxy = 0;
+    // upgrade
+    upgrade = pReq->IsUpgrade();
+
+    // proxy
+    proxy = forward_proxy = reverse_proxy = 0;
     if (hv::startswith(pReq->url, "http")) {
         // forward proxy
-        proxy = 1;
+        proxy = forward_proxy = 1;
+    }
+    else if (pReq->method == HTTP_CONNECT) {
+        // proxy tunnel
+        // CONNECT ip:port HTTP/1.1\r\n
+        pReq->url = "https://" + pReq->url;
+        proxy = forward_proxy = 1;
+        keepalive = true;
+    }
+
+    // printf("url=%s\n", pReq->url.c_str());
+    pReq->ParseUrl();
+
+    if (proxy) {
+        // Proxy-Connection
         auto iter = pReq->headers.find("Proxy-Connection");
         if (iter != pReq->headers.end()) {
             const char* keepalive_value = iter->second.c_str();
@@ -215,110 +378,42 @@ void HttpHandler::onHeadersComplete() {
             }
         }
     }
-
-    // printf("url=%s\n", pReq->url.c_str());
-    pReq->ParseUrl();
-
-    if (service->pathHandlers.size() != 0) {
-        service->GetRoute(pReq, &api_handler);
+    else {
+        // reverse proxy
+        std::string proxy_url = service->GetProxyUrl(pReq->path.c_str());
+        if (!proxy_url.empty()) {
+            pReq->url = proxy_url;
+            proxy = reverse_proxy = 1;
+        }
     }
-    if (api_handler && api_handler->state_handler) {
-        writer->onclose = [this](){
-            // HP_ERROR
-            if (!parser->IsComplete()) {
-                if (api_handler && api_handler->state_handler) {
-                    api_handler->state_handler(getHttpContext(), HP_ERROR, NULL, 0);
-                }
-            }
-        };
-    } else {
-        if (proxy) {
-            // forward proxy
-            if (service->enable_forward_proxy) {
-                proxyConnect(pReq->url);
-            } else {
-                proxy = 0;
-                resp->status_code = HTTP_STATUS_FORBIDDEN;
-                hlogw("Forbidden to forward proxy %s", pReq->url.c_str());
-            }
-        }
-        else if (service->proxies.size() != 0) {
-            // reverse proxy
-            std::string proxy_url = service->GetProxyUrl(pReq->path.c_str());
-            if (!proxy_url.empty()) {
-                proxy = 1;
-                pReq->url = proxy_url;
-                proxyConnect(pReq->url);
-            }
-        }
-        else {
-            // TODO: rewrite
-        }
+
+    // TODO: rewrite url
+}
+
+void HttpHandler::handleExpect100() {
+    // Expect: 100-continue
+    auto iter = req->headers.find("Expect");
+    if (iter != req->headers.end() &&
+        stricmp(iter->second.c_str(), "100-continue") == 0) {
+        if (io) hio_write(io, HTTP_100_CONTINUE_RESPONSE, HTTP_100_CONTINUE_RESPONSE_LEN);
     }
 }
 
-void HttpHandler::onProxyConnect(hio_t* upstream_io) {
-    // printf("onProxyConnect\n");
-    HttpHandler* handler = (HttpHandler*)hevent_userdata(upstream_io);
-    hio_t* io = hio_get_upstream(upstream_io);
-    assert(handler != NULL && io != NULL);
+void HttpHandler::addResponseHeaders() {
+    HttpResponse* pResp = resp.get();
+    // Server:
+    static char s_Server[64] = {'\0'};
+    if (s_Server[0] == '\0') {
+        snprintf(s_Server, sizeof(s_Server), "httpd/%s", hv_version());
+    }
+    pResp->headers["Server"] = s_Server;
 
-    HttpRequest* req = handler->req.get();
-    // NOTE: send head + received body
-    req->headers.erase("Proxy-Connection");
-    req->headers["Connection"] = handler->keepalive ? "keep-alive" : "close";
-    req->headers["X-Real-IP"] = handler->ip;
-    std::string msg = req->Dump(true, true);
-    // printf("%s\n", msg.c_str());
-    hio_write(upstream_io, msg.c_str(), msg.size());
-
-    // NOTE: start recv body continue then upstream
-    hio_setcb_read(io, hio_write_upstream);
-    hio_read_start(io);
-    hio_setcb_read(upstream_io, hio_write_upstream);
-    hio_read_start(upstream_io);
-}
-
-int HttpHandler::proxyConnect(const std::string& strUrl) {
-    if (!writer) return ERR_NULL_POINTER;
-    hio_t* io = writer->io();
-    hloop_t* loop = hevent_loop(io);
-
-    HUrl url;
-    if (!url.parse(strUrl)) {
-        return ERR_PARSE;
-    }
-
-    hlogi("proxy_pass %s", strUrl.c_str());
-    hio_t* upstream_io = hio_create_socket(loop, url.host.c_str(), url.port, HIO_TYPE_TCP, HIO_CLIENT_SIDE);
-    if (upstream_io == NULL) {
-        hio_close_async(io);
-        return ERR_SOCKET;
-    }
-    if (url.scheme == "https") {
-        hio_enable_ssl(upstream_io);
-    }
-    hevent_set_userdata(upstream_io, this);
-    hio_setup_upstream(io, upstream_io);
-    hio_setcb_connect(upstream_io, HttpHandler::onProxyConnect);
-    hio_setcb_close(upstream_io, hio_close_upstream);
-    if (service->proxy_connect_timeout > 0) {
-        hio_set_connect_timeout(upstream_io, service->proxy_connect_timeout);
-    }
-    if (service->proxy_read_timeout > 0) {
-        hio_set_read_timeout(io, service->proxy_read_timeout);
-    }
-    if (service->proxy_write_timeout > 0) {
-        hio_set_write_timeout(io, service->proxy_write_timeout);
-    }
-    hio_connect(upstream_io);
-    // NOTE: wait upstream_io connected then start read
-    hio_read_stop(io);
-    return 0;
+    // Connection:
+    pResp->headers["Connection"] = keepalive ? "keep-alive" : "close";
 }
 
 int HttpHandler::HandleHttpRequest() {
-    // preprocessor -> processor -> postprocessor
+    // preprocessor -> middleware -> processor -> postprocessor
     HttpRequest* pReq = req.get();
     HttpResponse* pResp = resp.get();
 
@@ -383,7 +478,6 @@ postprocessor:
         state = HANDLE_CONTINUE;
     } else {
         state = HANDLE_END;
-        parser->SubmitResponse(resp.get());
     }
     return status_code;
 }
@@ -419,7 +513,7 @@ int HttpHandler::defaultStaticHandler() {
     std::string path = req->Path();
     const char* req_path = path.c_str();
     // path safe check
-    if (req_path[0] != '/' || strstr(req_path, "/../")) {
+    if (req_path[0] != '/' || strstr(req_path, "/..") || strstr(req_path, "\\..")) {
         return HTTP_STATUS_BAD_REQUEST;
     }
 
@@ -478,7 +572,9 @@ int HttpHandler::defaultStaticHandler() {
     param.max_read = service->max_file_cache_size;
     param.need_read = !(req->method == HTTP_HEAD || has_range);
     param.path = req_path;
-    fc = files->Open(filepath.c_str(), &param);
+    if (files) {
+        fc = files->Open(filepath.c_str(), &param);
+    }
     if (fc == NULL) {
         if (param.error == ERR_OVER_LIMIT) {
             if (service->largeFileHandler) {
@@ -551,9 +647,11 @@ int HttpHandler::defaultErrorHandler() {
     // error page
     if (service->error_page.size() != 0) {
         std::string filepath = service->document_root + '/' + service->error_page;
-        // cache and load error page
-        FileCache::OpenParam param;
-        fc = files->Open(filepath.c_str(), &param);
+        if (files) {
+            // cache and load error page
+            FileCache::OpenParam param;
+            fc = files->Open(filepath.c_str(), &param);
+        }
     }
     // status page
     if (fc == NULL && resp->body.size() == 0) {
@@ -564,22 +662,65 @@ int HttpHandler::defaultErrorHandler() {
 }
 
 int HttpHandler::FeedRecvData(const char* data, size_t len) {
-    int nfeed = 0;
-    if (protocol == HttpHandler::WEBSOCKET) {
-        nfeed = ws_parser->FeedRecvData(data, len);
-        if (nfeed != len) {
-            hloge("[%s:%d] websocket parse error!", ip, port);
+    if (protocol == HttpHandler::UNKNOWN) {
+        int http_version = 1;
+#if WITH_NGHTTP2
+        if (strncmp(data, HTTP2_MAGIC, MIN(len, HTTP2_MAGIC_LEN)) == 0) {
+            http_version = 2;
         }
-    } else {
+#else
+        // check request-line
+        if (len < MIN_HTTP_REQUEST_LEN) {
+            hloge("[%s:%d] http request-line too small", ip, port);
+            error = ERR_REQUEST;
+            return -1;
+        }
+        for (int i = 0; i < MIN_HTTP_REQUEST_LEN; ++i) {
+            if (!IS_GRAPH(data[i])) {
+                hloge("[%s:%d] http request-line not plain", ip, port);
+                error = ERR_REQUEST;
+                return -1;
+            }
+        }
+#endif
+        if (!Init(http_version)) {
+            hloge("[%s:%d] unsupported HTTP%d", ip, port, http_version);
+            error = ERR_INVALID_PROTOCOL;
+            return -1;
+        }
+    }
+
+    int nfeed = 0;
+    switch (protocol) {
+    case HttpHandler::HTTP_V1:
+    case HttpHandler::HTTP_V2:
         if (state != WANT_RECV) {
             Reset();
         }
         nfeed = parser->FeedRecvData(data, len);
+        // printf("FeedRecvData %d=>%d\n", (int)len, nfeed);
         if (nfeed != len) {
             hloge("[%s:%d] http parse error: %s", ip, port, parser->StrError(parser->GetError()));
+            error = ERR_PARSE;
+            return -1;
         }
+        break;
+    case HttpHandler::WEBSOCKET:
+        nfeed = ws_parser->FeedRecvData(data, len);
+        if (nfeed != len) {
+            hloge("[%s:%d] websocket parse error!", ip, port);
+            error = ERR_PARSE;
+            return -1;
+        }
+        break;
+    default:
+        hloge("[%s:%d] unknown protocol", ip, port);
+        error = ERR_INVALID_PROTOCOL;
+        return -1;
     }
-    return nfeed;
+
+    if (state == WANT_CLOSE) return 0;
+    return error ? -1 : nfeed;
 }
 
 int HttpHandler::GetSendData(char** data, size_t* len) {
@@ -673,11 +814,38 @@ return_header:
             return 0;
         }
     } else if (protocol == HTTP_V2) {
-        return parser->GetSendData(data, len);
+        int ret = parser->GetSendData(data, len);
+        if (ret == 0) state = SEND_DONE;
+        return ret;
     }
     return 0;
 }
 
+int HttpHandler::SendHttpResponse(bool submit) {
+    if (!io || !parser) return -1;
+    char* data = NULL;
+    size_t len = 0, total_len = 0;
+    if (submit) parser->SubmitResponse(resp.get());
+    while (GetSendData(&data, &len)) {
+        // printf("GetSendData %d\n", (int)len);
+        if (data && len) {
+            hio_write(io, data, len);
+            total_len += len;
+        }
+    }
+    return total_len;
+}
+
+int HttpHandler::SendHttpStatusResponse(http_status status_code) {
+    if (state > WANT_SEND) return 0;
+    resp->status_code = status_code;
+    addResponseHeaders();
+    HandleHttpRequest();
+    state = WANT_SEND;
+    return SendHttpResponse();
+}
+
+//------------------sendfile--------------------------------------
 int HttpHandler::openFile(const char* filepath) {
     closeFile();
     file = new LargeFile;
@@ -701,6 +869,7 @@ int HttpHandler::sendFile() {
     size_t nread = file->read(file->buf.base, readbytes);
     if (nread <= 0) {
         hloge("read file error!");
+        error = ERR_READ_FILE;
         writer->close(true);
         return nread;
     }
@@ -728,3 +897,239 @@ void HttpHandler::closeFile() {
         file = NULL;
     }
 }
+
+//------------------upgrade--------------------------------------
+int HttpHandler::handleUpgrade(const char* upgrade_protocol) {
+    hlogi("[%s:%d] Upgrade: %s", ip, port, upgrade_protocol);
+
+    // websocket
+    if (stricmp(upgrade_protocol, "websocket") == 0) {
+        return upgradeWebSocket();
+    }
+
+    // h2/h2c
+    if (strnicmp(upgrade_protocol, "h2", 2) == 0) {
+        return upgradeHTTP2();
+    }
+
+    hloge("[%s:%d] unsupported Upgrade: %s", ip, port, upgrade_protocol);
+    return SetError(ERR_INVALID_PROTOCOL);
+}
+
+int HttpHandler::upgradeWebSocket() {
+    /*
+    HTTP/1.1 101 Switching Protocols
+    Connection: Upgrade
+    Upgrade: websocket
+    Sec-WebSocket-Accept: s3pPLMBiTxaQ9kYGzzhZRbK+xOo=
+    */
+    resp->status_code = HTTP_STATUS_SWITCHING_PROTOCOLS;
+    resp->headers["Connection"] = "Upgrade";
+    resp->headers["Upgrade"] = "websocket";
+
+    // Sec-WebSocket-Accept:
+    auto iter_key = req->headers.find(SEC_WEBSOCKET_KEY);
+    if (iter_key != req->headers.end()) {
+        char ws_accept[32] = {0};
+        ws_encode_key(iter_key->second.c_str(), ws_accept);
+        resp->headers[SEC_WEBSOCKET_ACCEPT] = ws_accept;
+    }
+
+    // Sec-WebSocket-Protocol:
+    auto iter_protocol = req->headers.find(SEC_WEBSOCKET_PROTOCOL);
+    if (iter_protocol != req->headers.end()) {
+        hv::StringList subprotocols = hv::split(iter_protocol->second, ',');
+        if (subprotocols.size() > 0) {
+            hlogw("%s: %s => just select first protocol %s", SEC_WEBSOCKET_PROTOCOL, iter_protocol->second.c_str(), subprotocols[0].c_str());
+            resp->headers[SEC_WEBSOCKET_PROTOCOL] = subprotocols[0];
+        }
+    }
+
+    SendHttpResponse();
+
+    if (!SwitchWebSocket()) {
+        hloge("[%s:%d] unsupported websocket", ip, port);
+        return SetError(ERR_INVALID_PROTOCOL);
+    }
+
+    // onopen
+    WebSocketOnOpen();
+    return 0;
+}
+
+int HttpHandler::upgradeHTTP2() {
+    /*
+    HTTP/1.1 101 Switching Protocols
+    Connection: Upgrade
+    Upgrade: h2c
+    */
+    resp->status_code = HTTP_STATUS_SWITCHING_PROTOCOLS;
+    resp->headers["Connection"] = "Upgrade";
+    resp->headers["Upgrade"] = "h2c";
+
+    SendHttpResponse();
+
+    if (!SwitchHTTP2()) {
+        hloge("[%s:%d] unsupported HTTP2", ip, port);
+        return SetError(ERR_INVALID_PROTOCOL);
+    }
+
+    // NOTE: send HTTP2_SETTINGS frame
+    SendHttpResponse(false);
+
+    return 0;
+}
+
+//------------------proxy--------------------------------------
+int HttpHandler::handleProxy() {
+    if (forward_proxy) {
+        return handleForwardProxy();
+    }
+
+    if (reverse_proxy) {
+        return handleReverseProxy();
+    }
+
+    return 0;
+}
+
+int HttpHandler::handleForwardProxy() {
+    if (service && service->enable_forward_proxy) {
+        return connectProxy(req->url);
+    } else {
+        hlogw("Forbidden to forward proxy %s", req->url.c_str());
+        SetError(HTTP_STATUS_FORBIDDEN, HTTP_STATUS_FORBIDDEN);
+    }
+    return 0;
+}
+
+int HttpHandler::handleReverseProxy() {
+    return connectProxy(req->url);
+}
+
+int HttpHandler::connectProxy(const std::string& strUrl) {
+    if (!io) return ERR_NULL_POINTER;
+
+    HUrl url;
+    url.parse(strUrl);
+    hlogi("[%s:%d] proxy_pass %s", ip, port, strUrl.c_str());
+
+    if (proxy_connected) {
+        if (url.host == proxy_host && url.port == proxy_port) {
+            // reuse keepalive connection
+            sendProxyRequest();
+            return 0;
+        } else {
+            // detach and close previous connection
+            hio_t* upstream_io = hio_get_upstream(io);
+            if (upstream_io) {
+                hio_setcb_close(upstream_io, NULL);
+                closeProxy();
+            }
+        }
+    }
+
+    if (forward_proxy && !service->IsTrustProxy(url.host.c_str())) {
+        hlogw("Forbidden to proxy %s", url.host.c_str());
+        SetError(HTTP_STATUS_FORBIDDEN, HTTP_STATUS_FORBIDDEN);
+        return 0;
+    }
+
+    hloop_t* loop = hevent_loop(io);
+    proxy = 1;
+    proxy_host = url.host;
+    proxy_port = url.port;
+    hio_t* upstream_io = hio_create_socket(loop, proxy_host.c_str(), proxy_port, HIO_TYPE_TCP, HIO_CLIENT_SIDE);
+    if (upstream_io == NULL) {
+        return SetError(ERR_SOCKET, HTTP_STATUS_BAD_GATEWAY);
+    }
+    if (url.scheme == "https") {
+        hio_enable_ssl(upstream_io);
+    }
+    hevent_set_userdata(upstream_io, this);
+    hio_setup_upstream(io, upstream_io);
+    hio_setcb_connect(upstream_io, HttpHandler::onProxyConnect);
+    hio_setcb_close(upstream_io, HttpHandler::onProxyClose);
+    if (service->proxy_connect_timeout > 0) {
+        hio_set_connect_timeout(upstream_io, service->proxy_connect_timeout);
+    }
+    if (service->proxy_read_timeout > 0) {
+        hio_set_read_timeout(io, service->proxy_read_timeout);
+    }
+    if (service->proxy_write_timeout > 0) {
+        hio_set_write_timeout(io, service->proxy_write_timeout);
+    }
+    hio_connect(upstream_io);
+    // NOTE: wait upstream_io connected then start read
+    hio_read_stop(io);
+    return 0;
+}
+
+int HttpHandler::closeProxy() {
+    if (proxy && proxy_connected) {
+        proxy_connected = 0;
+        if (io) hio_close_upstream(io);
+    }
+    return 0;
+}
+
+int HttpHandler::sendProxyRequest() {
+    if (!io || !proxy_connected) return -1;
+
+    req->headers.erase("Host");
+    req->FillHost(proxy_host.c_str(), proxy_port);
+    req->headers.erase("Proxy-Connection");
+    req->headers["Connection"] = keepalive ? "keep-alive" : "close";
+    req->headers["X-Real-IP"] = ip;
+    // NOTE: send head + received body
+    std::string msg = req->Dump(true, true);
+    // printf("%s\n", msg.c_str());
+    req->Reset();
+
+    hio_write_upstream(io, (void*)msg.c_str(), msg.size());
+    if (parser->IsComplete()) state = WANT_SEND;
+    return msg.size();
+}
+
+void HttpHandler::onProxyConnect(hio_t* upstream_io) {
+    // printf("onProxyConnect\n");
+    HttpHandler* handler = (HttpHandler*)hevent_userdata(upstream_io);
+    hio_t* io = hio_get_upstream(upstream_io);
+    assert(handler != NULL && io != NULL);
+    handler->proxy_connected = 1;
+
+    if (handler->req->method == HTTP_CONNECT) {
+        // handler->resp->status_code = HTTP_STATUS_OK;
+        // handler->SendHttpResponse();
+        hio_write(io, HTTP_200_CONNECT_RESPONSE, HTTP_200_CONNECT_RESPONSE_LEN);
+        handler->state = SEND_DONE;
+        // NOTE: recv request then upstream
+        hio_setcb_read(io, hio_write_upstream);
+    } else {
+        handler->sendProxyRequest();
+    }
+
+    // NOTE: start recv request continue then upstream
+    if (handler->upgrade) hio_setcb_read(io, hio_write_upstream);
+    hio_read_start(io);
+    // NOTE: start recv response then upstream
+    hio_setcb_read(upstream_io, hio_write_upstream);
+    hio_read_start(upstream_io);
+}
+
+void HttpHandler::onProxyClose(hio_t* upstream_io) {
+    // printf("onProxyClose\n");
+    HttpHandler* handler = (HttpHandler*)hevent_userdata(upstream_io);
+    if (handler == NULL) return;
+    handler->proxy_connected = 0;
+
+    hevent_set_userdata(upstream_io, NULL);
+
+    int error = hio_error(upstream_io);
+    if (error == ETIMEDOUT) {
+        handler->SendHttpStatusResponse(HTTP_STATUS_GATEWAY_TIMEOUT);
+    }
+
+    handler->error = error;
+    hio_close_upstream(upstream_io);
+}
